@@ -17,24 +17,39 @@ to_curvilinear = function(x) {
 }
 
 transform_curvilinear = function(x, crs, ...) {
-	if(is.numeric(crs))
+	if (is.numeric(crs))
 		crs = st_crs(crs)  	 # nocov
-	
-	if(inherits(crs, "crs"))
-		crs = crs$proj4string	
-		
+
+	to = crs
+	from = if (sf_extSoftVersion()["proj.4"] < "5.0.0") {
+			if (inherits(crs, "crs"))
+				to = crs$proj4string
+			else
+				stopifnot(is.character(crs))
+			st_crs(x)$proj4string
+		} else
+			st_crs(x)
+
+	get_yx = function(x) isTRUE(st_crs(st_sfc(st_point(), crs = x), parameters=TRUE)$yx)
+
 	d = st_dimensions(x)
 	xy = attr(d, "raster")$dimensions
 	cc = cbind(as.vector(d[[ xy[1] ]]$values), as.vector(d[[ xy[2] ]]$values))
-	pts = sf_project(from = d[[ xy[1] ]]$refsys, to = crs, pts = cc)
-	d[[ xy[1] ]]$refsys = d[[ xy[2] ]]$refsys = crs
+	pts = sf_project(from, to, cc)
+	d[[ xy[1] ]]$refsys = d[[ xy[2] ]]$refsys = st_crs(crs)
 	d[[ xy[1] ]]$values = matrix(pts[,1], dim(x)[xy])
 	d[[ xy[2] ]]$values = matrix(pts[,2], dim(x)[xy])
+	if (st_axis_order() && get_yx(st_crs(from)) != get_yx(st_crs(crs))) {
+		message("swapping [x] and [y] roles")
+		attr(d, "raster")$dimensions = rev(attr(d, "raster")$dimensions)
+		d[[ xy[1] ]]$values = t(d[[ xy[1] ]]$values)
+		d[[ xy[2] ]]$values = t(d[[ xy[2] ]]$values)
+	}
 	st_stars(x, d)
 }
 
 
-#' transform features, or warp/resample grids in stars objects to a new coordinate reference system
+#' transform geometries in stars objects to a new coordinate reference system, without warping
 #'
 #' @name st_transform
 #' @param x object of class \code{stars}, with either raster or simple feature geometries
@@ -51,7 +66,7 @@ transform_curvilinear = function(x, crs, ...) {
 #' plot(st_as_sfc(y, as_points=TRUE), pch=3, cex=.5, col = 'blue', add = TRUE)
 #' plot(st_transform(st_as_sfc(x, as_points=FALSE), new), add = TRUE)
 #' @seealso \link{st_warp}
-#' @details For simple feature dimensions, \link[sf]{st_transform} is called, leading to lossless transformation. For gridded spatial data, a curvilinear grid with transformed grid cell (centers) is returned. To convert this to a regular grid in the new \code{CRS}, use \link{st_warp}.
+#' @details For simple feature dimensions, \link[sf]{st_transform} is called, leading to lossless transformation. For gridded spatial data, a curvilinear grid with transformed grid cell (centers) is returned, which is also lossless. To convert this to a regular grid in the new \code{CRS}, use \link{st_warp} (which is in general lossy).
 #' @export
 st_transform.stars =  function(x, crs, ...) {
 
@@ -64,7 +79,7 @@ st_transform.stars =  function(x, crs, ...) {
 		ix = which_sfc(x)
 		for (j in ix) {
 			d[[j]]$values = st_transform(d[[j]]$values, crs, ...)
-			d[[j]]$refsys = crs$proj4string
+			d[[j]]$refsys = crs
 		}
 		structure(x, dimensions = d)
 	} else {
@@ -82,15 +97,22 @@ st_transform_proj.stars =  function(x, crs, ...) {
 
 	stopifnot(!is.na(crs), !is.na(st_crs(x)))
 
-	if (inherits(crs, "crs"))
-		crs = crs$proj4string
 	if (has_sfc(x)) {
     	if (!requireNamespace("lwgeom", quietly = TRUE))
         	stop("package lwgeom required, please install it first") # nocov
+		try_proj = function(x, crs) {
+			ret = try(st_transform(x, crs), silent = TRUE)
+			if (inherits(ret, "try-error")) {
+				if (inherits(crs, "crs"))
+					crs = crs$proj4string
+				ret = lwgeom::st_transform_proj(x, crs)
+			}
+			ret
+		}
 		d = st_dimensions(x)
 		ix = which_sfc(x)
 		for (j in ix) {
-			d[[j]]$values = lwgeom::st_transform_proj(d[[j]]$values, crs, ...)
+			d[[j]]$values = try_proj(d[[j]]$values, crs)
 			d[[j]]$refsys = crs
 		}
 		structure(x, dimensions = d)
