@@ -9,7 +9,9 @@ xx = as(x, "STFDF")
 
 # as.xts.stars:
 library(xts)
-as.xts(x)
+xts = as.xts(x)
+st_as_stars(xts)
+st_as_stars(xts, dimensions = st_dimensions(x))
 
 library(sp)
 pts = SpatialPoints(rbind(c(0,0), c(0,1), c(1,0), c(1,1)))
@@ -20,3 +22,5 @@ stfdf = STFDF(pts, tm, data.frame(foo = seq_len(length(pts) * length(tm))))
 x = st_as_stars(stfdf)
 xx = as(x, "STFDF")
 
+x = st_as_stars(as(stfdf, "STSDF"))
+
